<?php declare(strict_types=1);

namespace App\Services\DocumentationContent;

use App\Dto\Service\Admin\Project\DocumentationContent\Content;
use App\Dto\Service\Admin\Project\DocumentationContent\Contents;
use App\Exceptions\Services\DocumentationContent\ContentSaveException;
use App\Models\Documentation;
use App\Models\Project;

final readonly class ModelSyncCommand
{
    public function execute(Project $project, Documentation $documentation, Contents $contents): void
    {
        $languages = $project->languages;
        $documentationContents = $documentation->contents;

        $newContents = [];
        foreach ($contents->getContents() as $content)   {
            /** @var Content $content */
            $language = $languages->firstWhere('id', $content->getLanguageId());
            if (!$language) {
                throw new ContentSaveException('Language not found: ' . $content->getLanguageId());
            }

            $model = $documentationContents->firstWhere('language_id', $language->id);
            $data  = $this->getData($content);
            if (\is_null($model)) {
                $newContents[] = array_merge(['language_id' => $content->getLanguageId()], $data);
                continue;
            }
            $model->update($data);
        }

        if (!empty($newContents)) {
            $documentation->contents()->createMany($newContents);
        }
    }

    private function getData(Content $content): array
    {
        return [
            'title'   => $content->getTitle(),
            'content' => $content->getContent(),
        ];
    }
}
