<?php declare(strict_types=1);

namespace App\Services\Site;

use App\Dto\Service\Site\Feedback\Send;
use App\Models\Project;
use App\Models\User;
use App\ServiceResults\ServiceResultError;
use App\ServiceResults\ServiceResultSuccess;
use App\Services\ProjectFeedback\ProjectFeedbackCommandHandler;
use App\Services\Service;
use App\Services\WebsiteTranslations;
use Illuminate\Support\Facades\DB;

final class FeedbackService extends Service
{
    public function __construct(
        private readonly ProjectFeedbackCommandHandler $feedbackCommandHandler,
    ) { }

    public function send(Send $send, Project $project, WebsiteTranslations $websiteTranslations, ?User $user = null): ServiceResultError | ServiceResultSuccess
    {
        try {
            DB::transaction(function () use ($send, $project, $user) {
                $data = $this->getDataFeedback($send);
                $data['user_id'] = $user?->id;
                $this->feedbackCommandHandler->handleStore($project, $data);
            });
        } catch (\Throwable $e) {
            report($e);
            return $this->errService($websiteTranslations->translate('Server Error'));
        }

        return $this->ok($websiteTranslations->translate('site.Message sent successfully'));
    }

    private function getDataFeedback(Send $send): array
    {
        $name = $send->getName();
        if ($name !== null) {
            $name = strip_tags($name);
        }
        $message = $send->getMessage();
        $message = strip_tags($message, ['<b>', '<strong>', '<p>', '<br>', '<ul>', '<li>', '<ol>', '<table>', '<tr>', '<td>', '<th>', '<i>',]);
        return [
            'name'       => $name,
            'email'      => $send->getEmail(),
            'message'    => $message,
            'ip'         => $send->getHttpUserData()->getClientIp(),
            'user_agent' => $send->getHttpUserData()->getUserAgent(),
        ];
    }
}
